import { useCallback, useEffect, useReducer, useRef } from 'react';
import { requestError, requestInit, requestSuccess } from './action-creators';
import { httpRequestReducer, initialState } from './state-reducer';
import fastCompare from 'react-fast-compare';
import { useHttpClient } from '../client';
import { useCompareCallback, useCompareMemo } from '../shared';
export var useHttpRequest = function (params) {
    /**
     * Grabs the "request" function from the http client.
     */
    var httpClientAbortableRequest = useHttpClient().abortableRequest;
    // The state of the request.
    var _a = useReducer(httpRequestReducer, initialState(params.initialData)), state = _a[0], dispatch = _a[1];
    /**
     * A ref telling whether the component is currently mounted or not.
     */
    var isMounted = useRef(false);
    /**
     * Safely dispatches an action by first checking the mounting state of the component.
     */
    var safelyDispatch = useCallback(function (action) {
        if (isMounted.current) {
            dispatch(action);
        }
    }, [dispatch]);
    /**
     * Gets the http params needed to perform the request using the http client related method.
     */
    var performHttpRequestParams = useCompareMemo(function () { return ({
        baseUrlOverride: params.baseUrlOverride,
        parser: params.parser,
        relativeUrl: params.relativeUrl,
        requestOptions: params.requestOptions,
    }); }, [params], fastCompare);
    /**
     * Performs the http request allowing to abort it.
     */
    var request = useCompareCallback(function () {
        safelyDispatch(requestInit());
        try {
            var _a = httpClientAbortableRequest(performHttpRequestParams), reqPromise = _a[0], abortController = _a[1];
            reqPromise.then(function (response) { return safelyDispatch(requestSuccess(response)); });
            return [reqPromise, abortController];
        }
        catch (error) {
            // Dispatch the action handling the errored request.
            safelyDispatch(requestError(error));
            throw error;
        }
    }, [httpClientAbortableRequest, performHttpRequestParams, safelyDispatch], fastCompare);
    /**
     * Keeps track of the mounting state of the component.
     */
    useEffect(function () {
        isMounted.current = true;
        var fetchOnBootstrap = params.fetchOnBootstrap;
        if (fetchOnBootstrap) {
            request();
        }
        return function () {
            isMounted.current = false;
        };
    }, [params, request]);
    return [state, request];
};
